(***********************************************************************)
(*                              Project Couverture                     *)
(*                                                                     *)
(* file: coverExpr.ml                                                  *)
(* authors: Adrien Jonquet, Philippe Wang, Alexis Darrasse             *)
(* licence: CeCIL-B                                                    *)
(***********************************************************************)

let cover_expression src exec_trace events =
  let rec glance_events statements nb_uncovered nb_ev tr = function
  | [] -> (statements, nb_uncovered, nb_ev)
  | ev :: ls ->
      (* Look only for events associated with the src file *)
      if (CoverUtils.get_pos ev).Lexing.pos_fname = src then
        let l, l' = CoverUtils.get_loc ev in
        match ev.Covevent.covev_status with
        | Covevent.Coverage_event_condition _
        | Covevent.Coverage_event_decision ->
            glance_events statements nb_uncovered (nb_ev + 1) tr ls
        | Covevent.Coverage_event_statement ->
            let res = {
              Coverage.event = ev;
              cnum_start = l.Lexing.pos_cnum;
              cnum_end = l'.Lexing.pos_cnum;
              status = Coverage.Covered
            } in
            if tr then
              glance_events (res :: statements) nb_uncovered (nb_ev + 1) tr ls
            else
              glance_events
                ({ res with Coverage.status = Coverage.Uncovered } :: statements)
                (nb_uncovered + 1) (nb_ev + 1) tr ls
      else (statements, nb_uncovered, nb_ev) in
  let (_, (statements, nb_uncovered, nb_ev)) =
    Array.fold_left
      (fun (i, (statements, nb_uncovered, nb_ev)) -> fun event ->
        (i + 1, glance_events statements nb_uncovered nb_ev exec_trace.(i) event))
      (0, ([], 0, 0)) events in
  (statements, {
    Coverage.src = src;
    nb_events = nb_ev;
    nb_uncovered_events = nb_uncovered
  })
