(***********************************************************************)
(*                              Project Couverture                     *)
(*                                                                     *)
(* file: pluginTrace.ml                                                *)
(* authors: Adrien Jonquet, Philippe Wang, Alexis Darrasse             *)
(* licence: CeCIL-B                                                    *)
(***********************************************************************)

(* TODO deal with the case of multiple VM instances *)

let trace = ref false
let trace_name = ref ""
let exec_trace = ref (Array.make 0 false)

let step vm instruction =
  (* record the current obj instruction for source coverage *)
  if !trace && (not !exec_trace.(vm.Vm.code_pointer)) then
    !exec_trace.(vm.Vm.code_pointer) <- true

let init data =
  if !trace then begin
    let code_size = Array.length data.Bytecode_loader.code_section in
    if (Sys.file_exists !trace_name) then begin
      let ci = open_in !trace_name in
      exec_trace := Marshal.from_channel ci;
      close_in ci;
      if (Array.length !exec_trace <> code_size) then
        failwith "Existing trace is of wrong size"
    end else
      (* initialize the execution trace *)
      exec_trace := Array.make code_size false
  end

let finalise () =
  if !trace then
    (* trace generation *)
    try begin
      let co = open_out !trace_name in
      Marshal.to_channel co !exec_trace [];
      close_out co
    end with
    | Sys_error s -> failwith ("pluginTrace.ml : "^s)

let _ =
  Plugin.register_plugin init step finalise
    ("-trace", Arg.String (fun s -> trace := true; trace_name := s),
     "file  write trace in file")
